"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
import pydantic
from typing_extensions import Annotated, TypedDict
from unkey_py import utils
from unkey_py.types import BaseModel


class ErrDeleteProtectedCode(str, Enum):
    r"""A machine readable error code."""

    DELETE_PROTECTED = "DELETE_PROTECTED"


class ErrDeleteProtectedErrorTypedDict(TypedDict):
    code: ErrDeleteProtectedCode
    r"""A machine readable error code."""
    docs: str
    r"""A link to our documentation with more details about this error code"""
    message: str
    r"""A human readable explanation of what went wrong"""
    request_id: str
    r"""Please always include the requestId in your error report"""


class ErrDeleteProtectedError(BaseModel):
    code: ErrDeleteProtectedCode
    r"""A machine readable error code."""

    docs: str
    r"""A link to our documentation with more details about this error code"""

    message: str
    r"""A human readable explanation of what went wrong"""

    request_id: Annotated[str, pydantic.Field(alias="requestId")]
    r"""Please always include the requestId in your error report"""


class ErrDeleteProtectedData(BaseModel):
    error: ErrDeleteProtectedError


class ErrDeleteProtected(Exception):
    data: ErrDeleteProtectedData

    def __init__(self, data: ErrDeleteProtectedData):
        self.data = data

    def __str__(self) -> str:
        return utils.marshal_json(self.data, ErrDeleteProtectedData)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict
from unkey_py.types import BaseModel


class DeleteOverrideRequestBodyTypedDict(TypedDict):
    identifier: str
    r"""Identifier of your user, this can be their userId, an email, an ip or anything else. Wildcards ( * ) can be used to match multiple identifiers, More info can be found at https://www.unkey.com/docs/ratelimiting/overrides#wildcard-rules"""
    namespace_id: NotRequired[str]
    r"""The id of the namespace. Either namespaceId or namespaceName must be provided"""
    namespace_name: NotRequired[str]
    r"""The name of the namespace. Namespaces group different limits together for better analytics. You might have a namespace for your public API and one for internal tRPC routes."""


class DeleteOverrideRequestBody(BaseModel):
    identifier: str
    r"""Identifier of your user, this can be their userId, an email, an ip or anything else. Wildcards ( * ) can be used to match multiple identifiers, More info can be found at https://www.unkey.com/docs/ratelimiting/overrides#wildcard-rules"""

    namespace_id: Annotated[Optional[str], pydantic.Field(alias="namespaceId")] = None
    r"""The id of the namespace. Either namespaceId or namespaceName must be provided"""

    namespace_name: Annotated[Optional[str], pydantic.Field(alias="namespaceName")] = (
        None
    )
    r"""The name of the namespace. Namespaces group different limits together for better analytics. You might have a namespace for your public API and one for internal tRPC routes."""


class DeleteOverrideResponseBodyTypedDict(TypedDict):
    r"""Successfully deleted a ratelimit override"""


class DeleteOverrideResponseBody(BaseModel):
    r"""Successfully deleted a ratelimit override"""


class DeleteOverrideResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[DeleteOverrideResponseBodyTypedDict]
    r"""Successfully deleted a ratelimit override"""


class DeleteOverrideResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[DeleteOverrideResponseBody] = None
    r"""Successfully deleted a ratelimit override"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
from .key import Key, KeyTypedDict
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict
from unkey_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from unkey_py.utils import FieldMetadata, QueryParamMetadata


class ListKeysRequestTypedDict(TypedDict):
    api_id: str
    limit: NotRequired[int]
    cursor: NotRequired[str]
    owner_id: NotRequired[str]
    external_id: NotRequired[str]
    decrypt: NotRequired[Nullable[bool]]
    revalidate_keys_cache: NotRequired[Nullable[bool]]


class ListKeysRequest(BaseModel):
    api_id: Annotated[
        str,
        pydantic.Field(alias="apiId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ]

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 100

    cursor: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    owner_id: Annotated[
        Optional[str],
        pydantic.Field(alias="ownerId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    external_id: Annotated[
        Optional[str],
        pydantic.Field(alias="externalId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    decrypt: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    revalidate_keys_cache: Annotated[
        OptionalNullable[bool],
        pydantic.Field(alias="revalidateKeysCache"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = False

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "limit",
            "cursor",
            "ownerId",
            "externalId",
            "decrypt",
            "revalidateKeysCache",
        ]
        nullable_fields = ["decrypt", "revalidateKeysCache"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ListKeysResponseBodyTypedDict(TypedDict):
    r"""List of keys for the api"""

    keys: List[KeyTypedDict]
    total: int
    r"""The total number of keys for this api. This is an approximation and may lag behind up to 5 minutes."""
    cursor: NotRequired[str]
    r"""The cursor to use for the next page of results, if no cursor is returned, there are no more results"""


class ListKeysResponseBody(BaseModel):
    r"""List of keys for the api"""

    keys: List[Key]

    total: int
    r"""The total number of keys for this api. This is an approximation and may lag behind up to 5 minutes."""

    cursor: Optional[str] = None
    r"""The cursor to use for the next page of results, if no cursor is returned, there are no more results"""


class ListKeysResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[ListKeysResponseBodyTypedDict]
    r"""List of keys for the api"""


class ListKeysResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[ListKeysResponseBody] = None
    r"""List of keys for the api"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict
from unkey_py.types import BaseModel, Nullable


class WhoamiRequestBodyTypedDict(TypedDict):
    key: str
    r"""The actual key to fetch"""


class WhoamiRequestBody(BaseModel):
    key: str
    r"""The actual key to fetch"""


class WhoamiIdentityTypedDict(TypedDict):
    r"""The identity object associated with the key"""

    id: str
    r"""The identity ID associated with the key"""
    external_id: str
    r"""The external identity ID associated with the key"""


class WhoamiIdentity(BaseModel):
    r"""The identity object associated with the key"""

    id: str
    r"""The identity ID associated with the key"""

    external_id: Annotated[str, pydantic.Field(alias="externalId")]
    r"""The external identity ID associated with the key"""


class WhoamiResponseBodyTypedDict(TypedDict):
    r"""The configuration for a single key"""

    id: str
    r"""The ID of the key"""
    created_at: int
    r"""The timestamp in milliseconds when the key was created"""
    enabled: bool
    r"""Whether the key is enabled"""
    name: NotRequired[str]
    r"""The name of the key"""
    remaining: NotRequired[int]
    r"""The remaining number of requests for the key"""
    identity: NotRequired[WhoamiIdentityTypedDict]
    r"""The identity object associated with the key"""
    meta: NotRequired[Dict[str, Nullable[Any]]]
    r"""Metadata associated with the key"""
    environment: NotRequired[str]
    r"""The environment the key is associated with"""


class WhoamiResponseBody(BaseModel):
    r"""The configuration for a single key"""

    id: str
    r"""The ID of the key"""

    created_at: Annotated[int, pydantic.Field(alias="createdAt")]
    r"""The timestamp in milliseconds when the key was created"""

    enabled: bool
    r"""Whether the key is enabled"""

    name: Optional[str] = None
    r"""The name of the key"""

    remaining: Optional[int] = None
    r"""The remaining number of requests for the key"""

    identity: Optional[WhoamiIdentity] = None
    r"""The identity object associated with the key"""

    meta: Optional[Dict[str, Nullable[Any]]] = None
    r"""Metadata associated with the key"""

    environment: Optional[str] = None
    r"""The environment the key is associated with"""


class WhoamiResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[WhoamiResponseBodyTypedDict]
    r"""The configuration for a single key"""


class WhoamiResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[WhoamiResponseBody] = None
    r"""The configuration for a single key"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
from enum import Enum
import pydantic
from typing import Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict
from unkey_py.types import BaseModel


class Two(str, Enum):
    UNKNOWN = ""


DescriptionTypedDict = TypeAliasType("DescriptionTypedDict", Union[str, Two])
r"""Explain what this permission does. This is just for your team, your users will not see this."""


Description = TypeAliasType("Description", Union[str, Two])
r"""Explain what this permission does. This is just for your team, your users will not see this."""


class CreatePermissionRequestBodyTypedDict(TypedDict):
    name: str
    r"""The unique name of your permission."""
    description: NotRequired[DescriptionTypedDict]
    r"""Explain what this permission does. This is just for your team, your users will not see this."""


class CreatePermissionRequestBody(BaseModel):
    name: str
    r"""The unique name of your permission."""

    description: Optional[Description] = None
    r"""Explain what this permission does. This is just for your team, your users will not see this."""


class CreatePermissionResponseBodyTypedDict(TypedDict):
    r"""Sucessfully created a permission"""

    permission_id: str
    r"""The id of the permission. This is used internally"""


class CreatePermissionResponseBody(BaseModel):
    r"""Sucessfully created a permission"""

    permission_id: Annotated[str, pydantic.Field(alias="permissionId")]
    r"""The id of the permission. This is used internally"""


class CreatePermissionResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[CreatePermissionResponseBodyTypedDict]
    r"""Sucessfully created a permission"""


class CreatePermissionResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[CreatePermissionResponseBody] = None
    r"""Sucessfully created a permission"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict
from unkey_py.types import BaseModel


class DeleteKeyRequestBodyTypedDict(TypedDict):
    key_id: str
    r"""The id of the key to revoke"""
    permanent: NotRequired[bool]
    r"""By default Unkey soft deletes keys, so they may be recovered later. If you want to permanently delete it, set permanent=true. This might be necessary if you run into CONFLICT errors during key migration."""


class DeleteKeyRequestBody(BaseModel):
    key_id: Annotated[str, pydantic.Field(alias="keyId")]
    r"""The id of the key to revoke"""

    permanent: Optional[bool] = False
    r"""By default Unkey soft deletes keys, so they may be recovered later. If you want to permanently delete it, set permanent=true. This might be necessary if you run into CONFLICT errors during key migration."""


class DeleteKeyResponseBodyTypedDict(TypedDict):
    r"""The key was successfully revoked, it may take up to 30s for this to take effect in all regions"""


class DeleteKeyResponseBody(BaseModel):
    r"""The key was successfully revoked, it may take up to 30s for this to take effect in all regions"""


class DeleteKeyResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[DeleteKeyResponseBodyTypedDict]
    r"""The key was successfully revoked, it may take up to 30s for this to take effect in all regions"""


class DeleteKeyResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[DeleteKeyResponseBody] = None
    r"""The key was successfully revoked, it may take up to 30s for this to take effect in all regions"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict
from unkey_py.types import BaseModel


class ServicesTypedDict(TypedDict):
    metrics: str
    r"""The name of the connected metrics service"""
    logger: str
    r"""The name of the connected logger service"""
    ratelimit: str
    r"""The name of the connected ratelimit service"""
    usagelimit: str
    r"""The name of the connected usagelimit service"""


class Services(BaseModel):
    metrics: str
    r"""The name of the connected metrics service"""

    logger: str
    r"""The name of the connected logger service"""

    ratelimit: str
    r"""The name of the connected ratelimit service"""

    usagelimit: str
    r"""The name of the connected usagelimit service"""


class V1LivenessResponseBodyTypedDict(TypedDict):
    r"""The configured services and their status"""

    status: str
    r"""The status of the server"""
    services: ServicesTypedDict


class V1LivenessResponseBody(BaseModel):
    r"""The configured services and their status"""

    status: str
    r"""The status of the server"""

    services: Services


class V1LivenessResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[V1LivenessResponseBodyTypedDict]
    r"""The configured services and their status"""


class V1LivenessResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[V1LivenessResponseBody] = None
    r"""The configured services and their status"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
import pydantic
from typing import List, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict
from unkey_py.types import BaseModel, Nullable


class OrTypedDict(TypedDict):
    or_: List[Nullable[PermissionQueryTypedDict]]


class Or(BaseModel):
    or_: Annotated[List[Nullable[PermissionQuery]], pydantic.Field(alias="or")]


class AndTypedDict(TypedDict):
    and_: List[Nullable[PermissionQueryTypedDict]]


class And(BaseModel):
    and_: Annotated[List[Nullable[PermissionQuery]], pydantic.Field(alias="and")]


PermissionQueryTypedDict = TypeAliasType(
    "PermissionQueryTypedDict", Union["AndTypedDict", "OrTypedDict", str]
)
r"""A query for which permissions you require"""


PermissionQuery = TypeAliasType("PermissionQuery", Union["And", "Or", str])
r"""A query for which permissions you require"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict
from unkey_py.types import BaseModel


class DeleteRoleRequestBodyTypedDict(TypedDict):
    role_id: str
    r"""The id of the role you want to delete."""


class DeleteRoleRequestBody(BaseModel):
    role_id: Annotated[str, pydantic.Field(alias="roleId")]
    r"""The id of the role you want to delete."""


class DeleteRoleResponseBodyTypedDict(TypedDict):
    r"""Sucessfully deleted a role"""


class DeleteRoleResponseBody(BaseModel):
    r"""Sucessfully deleted a role"""


class DeleteRoleResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[DeleteRoleResponseBodyTypedDict]
    r"""Sucessfully deleted a role"""


class DeleteRoleResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[DeleteRoleResponseBody] = None
    r"""Sucessfully deleted a role"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
import pydantic
from typing import Any, Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict
from unkey_py.types import BaseModel, Nullable


class V1KeysVerifyKeyResponseRatelimitTypedDict(TypedDict):
    r"""The ratelimit configuration for this key. If this field is null or undefined, the key has no ratelimit."""

    limit: int
    r"""Maximum number of requests that can be made inside a window"""
    remaining: int
    r"""Remaining requests after this verification"""
    reset: int
    r"""Unix timestamp in milliseconds when the ratelimit will reset"""


class V1KeysVerifyKeyResponseRatelimit(BaseModel):
    r"""The ratelimit configuration for this key. If this field is null or undefined, the key has no ratelimit."""

    limit: int
    r"""Maximum number of requests that can be made inside a window"""

    remaining: int
    r"""Remaining requests after this verification"""

    reset: int
    r"""Unix timestamp in milliseconds when the ratelimit will reset"""


class Code(str, Enum):
    r"""A machine readable code why the key is not valid.
    Possible values are:
    - VALID: the key is valid and you should proceed
    - NOT_FOUND: the key does not exist or has expired
    - FORBIDDEN: the key is not allowed to access the api
    - USAGE_EXCEEDED: the key has exceeded its request limit
    - RATE_LIMITED: the key has been ratelimited
    - UNAUTHORIZED: the key is not authorized
    - DISABLED: the key is disabled
    - INSUFFICIENT_PERMISSIONS: you do not have the required permissions to perform this action
    - EXPIRED: The key was only valid for a certain time and has expired.

    These are validation codes, the HTTP status will be 200.

    """

    VALID = "VALID"
    NOT_FOUND = "NOT_FOUND"
    FORBIDDEN = "FORBIDDEN"
    USAGE_EXCEEDED = "USAGE_EXCEEDED"
    RATE_LIMITED = "RATE_LIMITED"
    UNAUTHORIZED = "UNAUTHORIZED"
    DISABLED = "DISABLED"
    INSUFFICIENT_PERMISSIONS = "INSUFFICIENT_PERMISSIONS"
    EXPIRED = "EXPIRED"


class V1KeysVerifyKeyResponseIdentityTypedDict(TypedDict):
    r"""The associated identity of this key."""

    id: str
    external_id: str
    meta: Dict[str, Nullable[Any]]


class V1KeysVerifyKeyResponseIdentity(BaseModel):
    r"""The associated identity of this key."""

    id: str

    external_id: Annotated[str, pydantic.Field(alias="externalId")]

    meta: Dict[str, Nullable[Any]]


class V1KeysVerifyKeyResponseTypedDict(TypedDict):
    valid: bool
    r"""Whether the key is valid or not.
    A key could be invalid for a number of reasons, for example if it has expired, has no more verifications left or if it has been deleted.
    """
    code: Code
    r"""A machine readable code why the key is not valid.
    Possible values are:
    - VALID: the key is valid and you should proceed
    - NOT_FOUND: the key does not exist or has expired
    - FORBIDDEN: the key is not allowed to access the api
    - USAGE_EXCEEDED: the key has exceeded its request limit
    - RATE_LIMITED: the key has been ratelimited
    - UNAUTHORIZED: the key is not authorized
    - DISABLED: the key is disabled
    - INSUFFICIENT_PERMISSIONS: you do not have the required permissions to perform this action
    - EXPIRED: The key was only valid for a certain time and has expired.

    These are validation codes, the HTTP status will be 200.

    """
    request_id: str
    r"""A unique id for this request, please provide it to Unkey support to help us debug your issue."""
    key_id: NotRequired[str]
    r"""The id of the key"""
    name: NotRequired[str]
    r"""The name of the key, give keys a name to easily identifiy their purpose"""
    owner_id: NotRequired[str]
    r"""The id of the tenant associated with this key. Use whatever reference you have in your system to identify the tenant. When verifying the key, we will send this field back to you, so you know who is accessing your API."""
    meta: NotRequired[Dict[str, Nullable[Any]]]
    r"""Any additional metadata you want to store with the key"""
    expires: NotRequired[int]
    r"""The unix timestamp in milliseconds when the key will expire. If this field is null or undefined, the key is not expiring."""
    ratelimit: NotRequired[V1KeysVerifyKeyResponseRatelimitTypedDict]
    r"""The ratelimit configuration for this key. If this field is null or undefined, the key has no ratelimit."""
    remaining: NotRequired[int]
    r"""The number of requests that can be made with this key before it becomes invalid. If this field is null or undefined, the key has no request limit."""
    enabled: NotRequired[bool]
    r"""Sets the key to be enabled or disabled. Disabled keys will not verify."""
    permissions: NotRequired[List[str]]
    r"""A list of all the permissions this key is connected to."""
    roles: NotRequired[List[str]]
    r"""A list of all the roles this key is connected to."""
    environment: NotRequired[str]
    r"""The environment of the key, this is what what you set when you crated the key"""
    identity: NotRequired[V1KeysVerifyKeyResponseIdentityTypedDict]
    r"""The associated identity of this key."""


class V1KeysVerifyKeyResponse(BaseModel):
    valid: bool
    r"""Whether the key is valid or not.
    A key could be invalid for a number of reasons, for example if it has expired, has no more verifications left or if it has been deleted.
    """

    code: Code
    r"""A machine readable code why the key is not valid.
    Possible values are:
    - VALID: the key is valid and you should proceed
    - NOT_FOUND: the key does not exist or has expired
    - FORBIDDEN: the key is not allowed to access the api
    - USAGE_EXCEEDED: the key has exceeded its request limit
    - RATE_LIMITED: the key has been ratelimited
    - UNAUTHORIZED: the key is not authorized
    - DISABLED: the key is disabled
    - INSUFFICIENT_PERMISSIONS: you do not have the required permissions to perform this action
    - EXPIRED: The key was only valid for a certain time and has expired.

    These are validation codes, the HTTP status will be 200.

    """

    request_id: Annotated[str, pydantic.Field(alias="requestId")]
    r"""A unique id for this request, please provide it to Unkey support to help us debug your issue."""

    key_id: Annotated[Optional[str], pydantic.Field(alias="keyId")] = None
    r"""The id of the key"""

    name: Optional[str] = None
    r"""The name of the key, give keys a name to easily identifiy their purpose"""

    owner_id: Annotated[Optional[str], pydantic.Field(alias="ownerId")] = None
    r"""The id of the tenant associated with this key. Use whatever reference you have in your system to identify the tenant. When verifying the key, we will send this field back to you, so you know who is accessing your API."""

    meta: Optional[Dict[str, Nullable[Any]]] = None
    r"""Any additional metadata you want to store with the key"""

    expires: Optional[int] = None
    r"""The unix timestamp in milliseconds when the key will expire. If this field is null or undefined, the key is not expiring."""

    ratelimit: Optional[V1KeysVerifyKeyResponseRatelimit] = None
    r"""The ratelimit configuration for this key. If this field is null or undefined, the key has no ratelimit."""

    remaining: Optional[int] = None
    r"""The number of requests that can be made with this key before it becomes invalid. If this field is null or undefined, the key has no request limit."""

    enabled: Optional[bool] = None
    r"""Sets the key to be enabled or disabled. Disabled keys will not verify."""

    permissions: Optional[List[str]] = None
    r"""A list of all the permissions this key is connected to."""

    roles: Optional[List[str]] = None
    r"""A list of all the roles this key is connected to."""

    environment: Optional[str] = None
    r"""The environment of the key, this is what what you set when you crated the key"""

    identity: Optional[V1KeysVerifyKeyResponseIdentity] = None
    r"""The associated identity of this key."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
from enum import Enum
import pydantic
from typing import Any, Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict, deprecated
from unkey_py.types import BaseModel, Nullable


class CreateKeyInterval(str, Enum):
    r"""Unkey will automatically refill verifications at the set interval."""

    DAILY = "daily"
    MONTHLY = "monthly"


class CreateKeyRefillTypedDict(TypedDict):
    r"""Unkey enables you to refill verifications for each key at regular intervals."""

    interval: CreateKeyInterval
    r"""Unkey will automatically refill verifications at the set interval."""
    amount: int
    r"""The number of verifications to refill for each occurrence is determined individually for each key."""
    refill_day: NotRequired[float]
    r"""The day of the month, when we will refill the remaining verifications. To refill on the 15th of each month, set 'refillDay': 15.
    If the day does not exist, for example you specified the 30th and it's february, we will refill them on the last day of the month instead.
    """


class CreateKeyRefill(BaseModel):
    r"""Unkey enables you to refill verifications for each key at regular intervals."""

    interval: CreateKeyInterval
    r"""Unkey will automatically refill verifications at the set interval."""

    amount: int
    r"""The number of verifications to refill for each occurrence is determined individually for each key."""

    refill_day: Annotated[Optional[float], pydantic.Field(alias="refillDay")] = None
    r"""The day of the month, when we will refill the remaining verifications. To refill on the 15th of each month, set 'refillDay': 15.
    If the day does not exist, for example you specified the 30th and it's february, we will refill them on the last day of the month instead.
    """


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class CreateKeyType(str, Enum):
    r"""Deprecated, use `async`. Fast ratelimiting doesn't add latency, while consistent ratelimiting is more accurate.
    https://unkey.dev/docs/features/ratelimiting - Learn more
    """

    FAST = "fast"
    CONSISTENT = "consistent"


class CreateKeyRatelimitTypedDict(TypedDict):
    r"""Unkey comes with per-key fixed-window ratelimiting out of the box."""

    limit: int
    r"""The total amount of requests in a given interval."""
    async_: NotRequired[bool]
    r"""Async will return a response immediately, lowering latency at the cost of accuracy. Will be required soon.
    https://unkey.dev/docs/features/ratelimiting - Learn more
    """
    type: NotRequired[CreateKeyType]
    r"""Deprecated, use `async`. Fast ratelimiting doesn't add latency, while consistent ratelimiting is more accurate.
    https://unkey.dev/docs/features/ratelimiting - Learn more
    """
    duration: NotRequired[int]
    r"""The window duration in milliseconds. Will be required soon."""
    refill_rate: NotRequired[int]
    r"""How many tokens to refill during each refillInterval."""
    refill_interval: NotRequired[int]
    r"""The refill timeframe, in milliseconds."""


class CreateKeyRatelimit(BaseModel):
    r"""Unkey comes with per-key fixed-window ratelimiting out of the box."""

    limit: int
    r"""The total amount of requests in a given interval."""

    async_: Annotated[Optional[bool], pydantic.Field(alias="async")] = True
    r"""Async will return a response immediately, lowering latency at the cost of accuracy. Will be required soon.
    https://unkey.dev/docs/features/ratelimiting - Learn more
    """

    type: Annotated[
        Optional[CreateKeyType],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = CreateKeyType.FAST
    r"""Deprecated, use `async`. Fast ratelimiting doesn't add latency, while consistent ratelimiting is more accurate.
    https://unkey.dev/docs/features/ratelimiting - Learn more
    """

    duration: Optional[int] = None
    r"""The window duration in milliseconds. Will be required soon."""

    refill_rate: Annotated[
        Optional[int],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible.",
            alias="refillRate",
        ),
    ] = None
    r"""How many tokens to refill during each refillInterval."""

    refill_interval: Annotated[
        Optional[int],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible.",
            alias="refillInterval",
        ),
    ] = None
    r"""The refill timeframe, in milliseconds."""


class CreateKeyRequestBodyTypedDict(TypedDict):
    api_id: str
    r"""Choose an `API` where this key should be created."""
    prefix: NotRequired[str]
    r"""To make it easier for your users to understand which product an api key belongs to, you can add prefix them.

    For example Stripe famously prefixes their customer ids with cus_ or their api keys with sk_live_.

    The underscore is automatically added if you are defining a prefix, for example: \"prefix\": \"abc\" will result in a key like abc_xxxxxxxxx

    """
    name: NotRequired[str]
    r"""The name for your Key. This is not customer facing."""
    byte_length: NotRequired[int]
    r"""The byte length used to generate your key determines its entropy as well as its length. Higher is better, but keys become longer and more annoying to handle. The default is 16 bytes, or 2^^128 possible combinations."""
    owner_id: NotRequired[str]
    r"""Deprecated, use `externalId`"""
    external_id: NotRequired[str]
    r"""Your user's Id. This will provide a link between Unkey and your customer record.
    When validating a key, we will return this back to you, so you can clearly identify your user from their api key.
    """
    meta: NotRequired[Dict[str, Nullable[Any]]]
    r"""This is a place for dynamic meta data, anything that feels useful for you should go here"""
    roles: NotRequired[List[str]]
    r"""A list of roles that this key should have. If the role does not exist, an error is thrown"""
    permissions: NotRequired[List[str]]
    r"""A list of permissions that this key should have. If the permission does not exist, an error is thrown"""
    expires: NotRequired[int]
    r"""You can auto expire keys by providing a unix timestamp in milliseconds. Once Keys expire they will automatically be disabled and are no longer valid unless you enable them again."""
    remaining: NotRequired[int]
    r"""You can limit the number of requests a key can make. Once a key reaches 0 remaining requests, it will automatically be disabled and is no longer valid unless you update it.
    https://unkey.dev/docs/features/remaining - Learn more
    """
    refill: NotRequired[CreateKeyRefillTypedDict]
    r"""Unkey enables you to refill verifications for each key at regular intervals."""
    ratelimit: NotRequired[CreateKeyRatelimitTypedDict]
    r"""Unkey comes with per-key fixed-window ratelimiting out of the box."""
    enabled: NotRequired[bool]
    r"""Sets if key is enabled or disabled. Disabled keys are not valid."""
    recoverable: NotRequired[bool]
    r"""You may want to show keys again later. While we do not recommend this, we leave this option open for you.

    In addition to storing the key's hash, recoverable keys are stored in an encrypted vault, allowing you to retrieve and display the plaintext later.

    [https://www.unkey.com/docs/security/recovering-keys](https://www.unkey.com/docs/security/recovering-keys) for more information.
    """
    environment: NotRequired[str]
    r"""Environments allow you to divide your keyspace.

    Some applications like Stripe, Clerk, WorkOS and others have a concept of \"live\" and \"test\" keys to
    give the developer a way to develop their own application without the risk of modifying real world
    resources.

    When you set an environment, we will return it back to you when validating the key, so you can
    handle it correctly.

    """


class CreateKeyRequestBody(BaseModel):
    api_id: Annotated[str, pydantic.Field(alias="apiId")]
    r"""Choose an `API` where this key should be created."""

    prefix: Optional[str] = None
    r"""To make it easier for your users to understand which product an api key belongs to, you can add prefix them.

    For example Stripe famously prefixes their customer ids with cus_ or their api keys with sk_live_.

    The underscore is automatically added if you are defining a prefix, for example: \"prefix\": \"abc\" will result in a key like abc_xxxxxxxxx

    """

    name: Optional[str] = None
    r"""The name for your Key. This is not customer facing."""

    byte_length: Annotated[Optional[int], pydantic.Field(alias="byteLength")] = 16
    r"""The byte length used to generate your key determines its entropy as well as its length. Higher is better, but keys become longer and more annoying to handle. The default is 16 bytes, or 2^^128 possible combinations."""

    owner_id: Annotated[
        Optional[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible.",
            alias="ownerId",
        ),
    ] = None
    r"""Deprecated, use `externalId`"""

    external_id: Annotated[Optional[str], pydantic.Field(alias="externalId")] = None
    r"""Your user's Id. This will provide a link between Unkey and your customer record.
    When validating a key, we will return this back to you, so you can clearly identify your user from their api key.
    """

    meta: Optional[Dict[str, Nullable[Any]]] = None
    r"""This is a place for dynamic meta data, anything that feels useful for you should go here"""

    roles: Optional[List[str]] = None
    r"""A list of roles that this key should have. If the role does not exist, an error is thrown"""

    permissions: Optional[List[str]] = None
    r"""A list of permissions that this key should have. If the permission does not exist, an error is thrown"""

    expires: Optional[int] = None
    r"""You can auto expire keys by providing a unix timestamp in milliseconds. Once Keys expire they will automatically be disabled and are no longer valid unless you enable them again."""

    remaining: Optional[int] = None
    r"""You can limit the number of requests a key can make. Once a key reaches 0 remaining requests, it will automatically be disabled and is no longer valid unless you update it.
    https://unkey.dev/docs/features/remaining - Learn more
    """

    refill: Optional[CreateKeyRefill] = None
    r"""Unkey enables you to refill verifications for each key at regular intervals."""

    ratelimit: Optional[CreateKeyRatelimit] = None
    r"""Unkey comes with per-key fixed-window ratelimiting out of the box."""

    enabled: Optional[bool] = True
    r"""Sets if key is enabled or disabled. Disabled keys are not valid."""

    recoverable: Optional[bool] = False
    r"""You may want to show keys again later. While we do not recommend this, we leave this option open for you.

    In addition to storing the key's hash, recoverable keys are stored in an encrypted vault, allowing you to retrieve and display the plaintext later.

    [https://www.unkey.com/docs/security/recovering-keys](https://www.unkey.com/docs/security/recovering-keys) for more information.
    """

    environment: Optional[str] = None
    r"""Environments allow you to divide your keyspace.

    Some applications like Stripe, Clerk, WorkOS and others have a concept of \"live\" and \"test\" keys to
    give the developer a way to develop their own application without the risk of modifying real world
    resources.

    When you set an environment, we will return it back to you when validating the key, so you can
    handle it correctly.

    """


class CreateKeyResponseBodyTypedDict(TypedDict):
    r"""The configuration for an api"""

    key_id: str
    r"""The id of the key. This is not a secret and can be stored as a reference if you wish. You need the keyId to update or delete a key later."""
    key: str
    r"""The newly created api key, do not store this on your own system but pass it along to your user."""


class CreateKeyResponseBody(BaseModel):
    r"""The configuration for an api"""

    key_id: Annotated[str, pydantic.Field(alias="keyId")]
    r"""The id of the key. This is not a secret and can be stored as a reference if you wish. You need the keyId to update or delete a key later."""

    key: str
    r"""The newly created api key, do not store this on your own system but pass it along to your user."""


class CreateKeyResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[CreateKeyResponseBodyTypedDict]
    r"""The configuration for an api"""


class CreateKeyResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[CreateKeyResponseBody] = None
    r"""The configuration for an api"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
from enum import Enum
import pydantic
from pydantic import model_serializer
from typing import List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict
from unkey_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from unkey_py.utils import FieldMetadata, QueryParamMetadata


KeyIDTypedDict = TypeAliasType("KeyIDTypedDict", Union[str, List[str]])
r"""Only include data for a specific key or keys.

When you are providing zero or more than one key ids, all usage counts are aggregated and summed up. Send multiple requests with one keyId each if you need counts per key.


"""


KeyID = TypeAliasType("KeyID", Union[str, List[str]])
r"""Only include data for a specific key or keys.

When you are providing zero or more than one key ids, all usage counts are aggregated and summed up. Send multiple requests with one keyId each if you need counts per key.


"""


TagTypedDict = TypeAliasType("TagTypedDict", Union[str, List[str]])
r"""Only include data for a specific tag or tags.

When you are providing zero or more than one tag, all usage counts are aggregated and summed up. Send multiple requests with one tag each if you need counts per tag.
"""


Tag = TypeAliasType("Tag", Union[str, List[str]])
r"""Only include data for a specific tag or tags.

When you are providing zero or more than one tag, all usage counts are aggregated and summed up. Send multiple requests with one tag each if you need counts per tag.
"""


class QueryParam2(str, Enum):
    KEY = "key"
    IDENTITY = "identity"
    TAGS = "tags"
    TAG = "tag"
    MONTH = "month"
    DAY = "day"
    HOUR = "hour"


class One(str, Enum):
    KEY = "key"
    IDENTITY = "identity"
    TAGS = "tags"
    TAG = "tag"
    MONTH = "month"
    DAY = "day"
    HOUR = "hour"


GroupByTypedDict = TypeAliasType("GroupByTypedDict", Union[One, List[QueryParam2]])
r"""By default, datapoints are not aggregated, however you probably want to get a breakdown per time, key or identity.

Grouping by tags and by tag is mutually exclusive.
"""


GroupBy = TypeAliasType("GroupBy", Union[One, List[QueryParam2]])
r"""By default, datapoints are not aggregated, however you probably want to get a breakdown per time, key or identity.

Grouping by tags and by tag is mutually exclusive.
"""


class OrderBy(str, Enum):
    r"""Sort the output by a specific value. You can use this in combination with the `order` param."""

    TIME = "time"
    VALID = "valid"
    NOT_FOUND = "notFound"
    FORBIDDEN = "forbidden"
    USAGE_EXCEEDED = "usageExceeded"
    RATE_LIMITED = "rateLimited"
    UNAUTHORIZED = "unauthorized"
    DISABLED = "disabled"
    INSUFFICIENT_PERMISSIONS = "insufficientPermissions"
    EXPIRED = "expired"
    TOTAL = "total"


class Order(str, Enum):
    r"""Define the order of sorting. Use this in combination with `orderBy`"""

    ASC = "asc"
    DESC = "desc"


class GetVerificationsRequestTypedDict(TypedDict):
    api_id: str
    external_id: NotRequired[str]
    key_id: NotRequired[KeyIDTypedDict]
    r"""Only include data for a specific key or keys.

    When you are providing zero or more than one key ids, all usage counts are aggregated and summed up. Send multiple requests with one keyId each if you need counts per key.


    """
    tag: NotRequired[TagTypedDict]
    r"""Only include data for a specific tag or tags.

    When you are providing zero or more than one tag, all usage counts are aggregated and summed up. Send multiple requests with one tag each if you need counts per tag.
    """
    start: NotRequired[Nullable[int]]
    end: NotRequired[Nullable[int]]
    group_by: NotRequired[GroupByTypedDict]
    r"""By default, datapoints are not aggregated, however you probably want to get a breakdown per time, key or identity.

    Grouping by tags and by tag is mutually exclusive.
    """
    limit: NotRequired[int]
    order_by: NotRequired[OrderBy]
    r"""Sort the output by a specific value. You can use this in combination with the `order` param."""
    order: NotRequired[Order]
    r"""Define the order of sorting. Use this in combination with `orderBy`"""


class GetVerificationsRequest(BaseModel):
    api_id: Annotated[
        str,
        pydantic.Field(alias="apiId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ]

    external_id: Annotated[
        Optional[str],
        pydantic.Field(alias="externalId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    key_id: Annotated[
        Optional[KeyID],
        pydantic.Field(alias="keyId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Only include data for a specific key or keys.

    When you are providing zero or more than one key ids, all usage counts are aggregated and summed up. Send multiple requests with one keyId each if you need counts per key.


    """

    tag: Annotated[
        Optional[Tag],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Only include data for a specific tag or tags.

    When you are providing zero or more than one tag, all usage counts are aggregated and summed up. Send multiple requests with one tag each if you need counts per tag.
    """

    start: Annotated[
        OptionalNullable[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    end: Annotated[
        OptionalNullable[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1746664158001

    group_by: Annotated[
        Optional[GroupBy],
        pydantic.Field(alias="groupBy"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""By default, datapoints are not aggregated, however you probably want to get a breakdown per time, key or identity.

    Grouping by tags and by tag is mutually exclusive.
    """

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    order_by: Annotated[
        Optional[OrderBy],
        pydantic.Field(alias="orderBy"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Sort the output by a specific value. You can use this in combination with the `order` param."""

    order: Annotated[
        Optional[Order],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = Order.ASC
    r"""Define the order of sorting. Use this in combination with `orderBy`"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "externalId",
            "keyId",
            "tag",
            "start",
            "end",
            "groupBy",
            "limit",
            "orderBy",
            "order",
        ]
        nullable_fields = ["start", "end"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetVerificationsIdentityTypedDict(TypedDict):
    r"""Only available when specifying groupBy=identity in the query.
    In this case there would be one datapoint per time and groupBy target.
    """

    id: str
    external_id: str


class GetVerificationsIdentity(BaseModel):
    r"""Only available when specifying groupBy=identity in the query.
    In this case there would be one datapoint per time and groupBy target.
    """

    id: str

    external_id: Annotated[str, pydantic.Field(alias="externalId")]


class GetVerificationsResponseBodyTypedDict(TypedDict):
    total: int
    r"""Total number of verifications in the current time slice, regardless of outcome."""
    time: NotRequired[int]
    r"""Unix timestamp in milliseconds of the start of the current time slice."""
    valid: NotRequired[int]
    not_found: NotRequired[int]
    forbidden: NotRequired[int]
    usage_exceeded: NotRequired[int]
    rate_limited: NotRequired[int]
    unauthorized: NotRequired[int]
    disabled: NotRequired[int]
    insufficient_permissions: NotRequired[int]
    expired: NotRequired[int]
    tag: NotRequired[str]
    r"""Only available when grouping by tag."""
    tags: NotRequired[List[str]]
    r"""Filter by one or multiple tags. If multiple tags are provided"""
    key_id: NotRequired[str]
    r"""Only available when specifying groupBy=key in the query.
    In this case there would be one datapoint per time and groupBy target.
    """
    identity: NotRequired[GetVerificationsIdentityTypedDict]
    r"""Only available when specifying groupBy=identity in the query.
    In this case there would be one datapoint per time and groupBy target.
    """


class GetVerificationsResponseBody(BaseModel):
    total: int
    r"""Total number of verifications in the current time slice, regardless of outcome."""

    time: Optional[int] = None
    r"""Unix timestamp in milliseconds of the start of the current time slice."""

    valid: Optional[int] = None

    not_found: Annotated[Optional[int], pydantic.Field(alias="notFound")] = None

    forbidden: Optional[int] = None

    usage_exceeded: Annotated[Optional[int], pydantic.Field(alias="usageExceeded")] = (
        None
    )

    rate_limited: Annotated[Optional[int], pydantic.Field(alias="rateLimited")] = None

    unauthorized: Optional[int] = None

    disabled: Optional[int] = None

    insufficient_permissions: Annotated[
        Optional[int], pydantic.Field(alias="insufficientPermissions")
    ] = None

    expired: Optional[int] = None

    tag: Optional[str] = None
    r"""Only available when grouping by tag."""

    tags: Optional[List[str]] = None
    r"""Filter by one or multiple tags. If multiple tags are provided"""

    key_id: Annotated[Optional[str], pydantic.Field(alias="keyId")] = None
    r"""Only available when specifying groupBy=key in the query.
    In this case there would be one datapoint per time and groupBy target.
    """

    identity: Optional[GetVerificationsIdentity] = None
    r"""Only available when specifying groupBy=identity in the query.
    In this case there would be one datapoint per time and groupBy target.
    """


class GetVerificationsResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    response_bodies: NotRequired[List[GetVerificationsResponseBodyTypedDict]]
    r"""Retrieve all required data to build end-user facing dashboards and drive your usage-based billing."""


class GetVerificationsResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    response_bodies: Optional[List[GetVerificationsResponseBody]] = None
    r"""Retrieve all required data to build end-user facing dashboards and drive your usage-based billing."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
from enum import Enum
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict
from unkey_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from unkey_py.utils import FieldMetadata, QueryParamMetadata


class Granularity(str, Enum):
    r"""The granularity of the usage data to fetch, currently only `day` is supported"""

    DAY = "day"


class KeysGetVerificationsRequestTypedDict(TypedDict):
    key_id: NotRequired[str]
    owner_id: NotRequired[str]
    start: NotRequired[Nullable[int]]
    end: NotRequired[Nullable[int]]
    granularity: NotRequired[Granularity]
    r"""The granularity of the usage data to fetch, currently only `day` is supported"""


class KeysGetVerificationsRequest(BaseModel):
    key_id: Annotated[
        Optional[str],
        pydantic.Field(alias="keyId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    owner_id: Annotated[
        Optional[str],
        pydantic.Field(alias="ownerId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    start: Annotated[
        OptionalNullable[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    end: Annotated[
        OptionalNullable[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET

    granularity: Annotated[
        Optional[Granularity],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = Granularity.DAY
    r"""The granularity of the usage data to fetch, currently only `day` is supported"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["keyId", "ownerId", "start", "end", "granularity"]
        nullable_fields = ["start", "end"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class VerificationsTypedDict(TypedDict):
    time: int
    r"""The timestamp of the usage data"""
    success: int
    r"""The number of successful requests"""
    rate_limited: int
    r"""The number of requests that were rate limited"""
    usage_exceeded: int
    r"""The number of requests that exceeded the usage limit"""


class Verifications(BaseModel):
    time: int
    r"""The timestamp of the usage data"""

    success: int
    r"""The number of successful requests"""

    rate_limited: Annotated[int, pydantic.Field(alias="rateLimited")]
    r"""The number of requests that were rate limited"""

    usage_exceeded: Annotated[int, pydantic.Field(alias="usageExceeded")]
    r"""The number of requests that exceeded the usage limit"""


class KeysGetVerificationsResponseBodyTypedDict(TypedDict):
    r"""Usage numbers over time"""

    verifications: List[VerificationsTypedDict]


class KeysGetVerificationsResponseBody(BaseModel):
    r"""Usage numbers over time"""

    verifications: List[Verifications]


class KeysGetVerificationsResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[KeysGetVerificationsResponseBodyTypedDict]
    r"""Usage numbers over time"""


class KeysGetVerificationsResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[KeysGetVerificationsResponseBody] = None
    r"""Usage numbers over time"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict
from unkey_py.types import BaseModel


class DeleteKeysRequestBodyTypedDict(TypedDict):
    api_id: str
    r"""The id of the api, that the keys belong to."""
    permanent: NotRequired[bool]
    r"""Delete the keys permanently, if false the keys will be marked as deleted but not removed from the database. In either case, the keys will no longer be valid when verifying them."""


class DeleteKeysRequestBody(BaseModel):
    api_id: Annotated[str, pydantic.Field(alias="apiId")]
    r"""The id of the api, that the keys belong to."""

    permanent: Optional[bool] = False
    r"""Delete the keys permanently, if false the keys will be marked as deleted but not removed from the database. In either case, the keys will no longer be valid when verifying them."""


class DeleteKeysResponseBodyTypedDict(TypedDict):
    r"""The keys have been deleted"""

    deleted_keys: int
    r"""The number of keys that were deleted"""


class DeleteKeysResponseBody(BaseModel):
    r"""The keys have been deleted"""

    deleted_keys: Annotated[int, pydantic.Field(alias="deletedKeys")]
    r"""The number of keys that were deleted"""


class DeleteKeysResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[DeleteKeysResponseBodyTypedDict]
    r"""The keys have been deleted"""


class DeleteKeysResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[DeleteKeysResponseBody] = None
    r"""The keys have been deleted"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict
from unkey_py.types import BaseModel


class AddRolesRolesTypedDict(TypedDict):
    id: NotRequired[str]
    r"""The id of the role. Provide either `id` or `name`. If both are provided `id` is used."""
    name: NotRequired[str]
    r"""Identify the role via its name. Provide either `id` or `name`. If both are provided `id` is used."""
    create: NotRequired[bool]
    r"""Set to true to automatically create the permissions they do not exist yet. Only works when specifying `name`.
    Autocreating roles requires your root key to have the `rbac.*.create_role` permission, otherwise the request will get rejected
    """


class AddRolesRoles(BaseModel):
    id: Optional[str] = None
    r"""The id of the role. Provide either `id` or `name`. If both are provided `id` is used."""

    name: Optional[str] = None
    r"""Identify the role via its name. Provide either `id` or `name`. If both are provided `id` is used."""

    create: Optional[bool] = None
    r"""Set to true to automatically create the permissions they do not exist yet. Only works when specifying `name`.
    Autocreating roles requires your root key to have the `rbac.*.create_role` permission, otherwise the request will get rejected
    """


class AddRolesRequestBodyTypedDict(TypedDict):
    key_id: str
    r"""The id of the key."""
    roles: List[AddRolesRolesTypedDict]
    r"""The roles you want to set for this key. This overwrites all existing roles.
    Setting roles requires the `rbac.*.add_role_to_key` permission.
    """


class AddRolesRequestBody(BaseModel):
    key_id: Annotated[str, pydantic.Field(alias="keyId")]
    r"""The id of the key."""

    roles: List[AddRolesRoles]
    r"""The roles you want to set for this key. This overwrites all existing roles.
    Setting roles requires the `rbac.*.add_role_to_key` permission.
    """


class AddRolesResponseBodyTypedDict(TypedDict):
    id: str
    r"""The id of the role. This is used internally"""
    name: str
    r"""The name of the role"""


class AddRolesResponseBody(BaseModel):
    id: str
    r"""The id of the role. This is used internally"""

    name: str
    r"""The name of the role"""


class AddRolesResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    response_bodies: NotRequired[List[AddRolesResponseBodyTypedDict]]
    r"""All currently connected roles"""


class AddRolesResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    response_bodies: Optional[List[AddRolesResponseBody]] = None
    r"""All currently connected roles"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict
from unkey_py.types import BaseModel


class ListPermissionsResponseBodyTypedDict(TypedDict):
    id: str
    r"""The id of the permission"""
    name: str
    r"""The name of the permission."""
    description: NotRequired[str]
    r"""The description of what this permission does. This is just for your team, your users will not see this."""


class ListPermissionsResponseBody(BaseModel):
    id: str
    r"""The id of the permission"""

    name: str
    r"""The name of the permission."""

    description: Optional[str] = None
    r"""The description of what this permission does. This is just for your team, your users will not see this."""


class ListPermissionsResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    response_bodies: NotRequired[List[ListPermissionsResponseBodyTypedDict]]
    r"""The permissions in your workspace"""


class ListPermissionsResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    response_bodies: Optional[List[ListPermissionsResponseBody]] = None
    r"""The permissions in your workspace"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
from enum import Enum
import pydantic
from typing import Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict
from unkey_py.types import BaseModel


class Description2(str, Enum):
    UNKNOWN = ""


CreateRoleDescriptionTypedDict = TypeAliasType(
    "CreateRoleDescriptionTypedDict", Union[str, Description2]
)
r"""Explain what this role does. This is just for your team, your users will not see this."""


CreateRoleDescription = TypeAliasType("CreateRoleDescription", Union[str, Description2])
r"""Explain what this role does. This is just for your team, your users will not see this."""


class CreateRoleRequestBodyTypedDict(TypedDict):
    name: str
    r"""The unique name of your role."""
    description: NotRequired[CreateRoleDescriptionTypedDict]
    r"""Explain what this role does. This is just for your team, your users will not see this."""


class CreateRoleRequestBody(BaseModel):
    name: str
    r"""The unique name of your role."""

    description: Optional[CreateRoleDescription] = None
    r"""Explain what this role does. This is just for your team, your users will not see this."""


class CreateRoleResponseBodyTypedDict(TypedDict):
    r"""Sucessfully created a role"""

    role_id: str
    r"""The id of the role. This is used internally"""


class CreateRoleResponseBody(BaseModel):
    r"""Sucessfully created a role"""

    role_id: Annotated[str, pydantic.Field(alias="roleId")]
    r"""The id of the role. This is used internally"""


class CreateRoleResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[CreateRoleResponseBodyTypedDict]
    r"""Sucessfully created a role"""


class CreateRoleResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[CreateRoleResponseBody] = None
    r"""Sucessfully created a role"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
from enum import Enum
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict
from unkey_py.types import BaseModel, Nullable, UNSET_SENTINEL


class Op(str, Enum):
    r"""The operation you want to perform on the remaining count"""

    INCREMENT = "increment"
    DECREMENT = "decrement"
    SET = "set"


class UpdateRemainingRequestBodyTypedDict(TypedDict):
    key_id: str
    r"""The id of the key you want to modify"""
    op: Op
    r"""The operation you want to perform on the remaining count"""
    value: Nullable[int]
    r"""The value you want to set, add or subtract the remaining count by"""


class UpdateRemainingRequestBody(BaseModel):
    key_id: Annotated[str, pydantic.Field(alias="keyId")]
    r"""The id of the key you want to modify"""

    op: Op
    r"""The operation you want to perform on the remaining count"""

    value: Nullable[int]
    r"""The value you want to set, add or subtract the remaining count by"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["value"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class UpdateRemainingResponseBodyTypedDict(TypedDict):
    r"""The configuration for an api"""

    remaining: Nullable[int]
    r"""The number of remaining requests for this key after updating it. `null` means unlimited."""


class UpdateRemainingResponseBody(BaseModel):
    r"""The configuration for an api"""

    remaining: Nullable[int]
    r"""The number of remaining requests for this key after updating it. `null` means unlimited."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["remaining"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class UpdateRemainingResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[UpdateRemainingResponseBodyTypedDict]
    r"""The configuration for an api"""


class UpdateRemainingResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[UpdateRemainingResponseBody] = None
    r"""The configuration for an api"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from typing import Any, Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict
from unkey_py.types import BaseModel, Nullable


class UpdateIdentityRatelimitsTypedDict(TypedDict):
    name: str
    r"""The name of this limit. You will need to use this again when verifying a key."""
    limit: int
    r"""How many requests may pass within a given window before requests are rejected."""
    duration: int
    r"""The duration for each ratelimit window in milliseconds."""


class UpdateIdentityRatelimits(BaseModel):
    name: str
    r"""The name of this limit. You will need to use this again when verifying a key."""

    limit: int
    r"""How many requests may pass within a given window before requests are rejected."""

    duration: int
    r"""The duration for each ratelimit window in milliseconds."""


class UpdateIdentityRequestBodyTypedDict(TypedDict):
    identity_id: NotRequired[str]
    r"""The id of the identity to update, use either `identityId` or `externalId`, if both are provided, `identityId` takes precedence."""
    external_id: NotRequired[str]
    r"""The externalId of the identity to update, use either `identityId` or `externalId`, if both are provided, `identityId` takes precedence."""
    environment: NotRequired[str]
    r"""This is not yet used but here for future compatibility."""
    meta: NotRequired[Dict[str, Nullable[Any]]]
    r"""Attach metadata to this identity that you need to have access to when verifying a key.

    Set to `{}` to clear.

    This will be returned as part of the `verifyKey` response.

    """
    ratelimits: NotRequired[List[UpdateIdentityRatelimitsTypedDict]]
    r"""Attach ratelimits to this identity.

    This overwrites all existing ratelimits on this identity.
    Setting an empty array will delete all existing ratelimits.

    When verifying keys, you can specify which limits you want to use and all keys attached to this identity, will share the limits.
    """


class UpdateIdentityRequestBody(BaseModel):
    identity_id: Annotated[Optional[str], pydantic.Field(alias="identityId")] = None
    r"""The id of the identity to update, use either `identityId` or `externalId`, if both are provided, `identityId` takes precedence."""

    external_id: Annotated[Optional[str], pydantic.Field(alias="externalId")] = None
    r"""The externalId of the identity to update, use either `identityId` or `externalId`, if both are provided, `identityId` takes precedence."""

    environment: Optional[str] = "default"
    r"""This is not yet used but here for future compatibility."""

    meta: Optional[Dict[str, Nullable[Any]]] = None
    r"""Attach metadata to this identity that you need to have access to when verifying a key.

    Set to `{}` to clear.

    This will be returned as part of the `verifyKey` response.

    """

    ratelimits: Optional[List[UpdateIdentityRatelimits]] = None
    r"""Attach ratelimits to this identity.

    This overwrites all existing ratelimits on this identity.
    Setting an empty array will delete all existing ratelimits.

    When verifying keys, you can specify which limits you want to use and all keys attached to this identity, will share the limits.
    """


class UpdateIdentityIdentitiesRatelimitsTypedDict(TypedDict):
    name: str
    r"""The name of this limit."""
    limit: int
    r"""How many requests may pass within a given window before requests are rejected."""
    duration: int
    r"""The duration for each ratelimit window in milliseconds."""


class UpdateIdentityIdentitiesRatelimits(BaseModel):
    name: str
    r"""The name of this limit."""

    limit: int
    r"""How many requests may pass within a given window before requests are rejected."""

    duration: int
    r"""The duration for each ratelimit window in milliseconds."""


class UpdateIdentityResponseBodyTypedDict(TypedDict):
    r"""The identity after the update."""

    id: str
    r"""The id of the identity."""
    external_id: str
    r"""The externalId of the identity."""
    meta: Dict[str, Nullable[Any]]
    r"""The metadata attached to this identity."""
    ratelimits: List[UpdateIdentityIdentitiesRatelimitsTypedDict]


class UpdateIdentityResponseBody(BaseModel):
    r"""The identity after the update."""

    id: str
    r"""The id of the identity."""

    external_id: Annotated[str, pydantic.Field(alias="externalId")]
    r"""The externalId of the identity."""

    meta: Dict[str, Nullable[Any]]
    r"""The metadata attached to this identity."""

    ratelimits: List[UpdateIdentityIdentitiesRatelimits]


class UpdateIdentityResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[UpdateIdentityResponseBodyTypedDict]
    r"""The identity after the update."""


class UpdateIdentityResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[UpdateIdentityResponseBody] = None
    r"""The identity after the update."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
from enum import Enum
import pydantic
from typing import Any, Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict, deprecated
from unkey_py.types import BaseModel, Nullable


class Variant(str, Enum):
    r"""The algorithm for hashing and encoding, currently only sha256 and base64 are supported"""

    SHA256_BASE64 = "sha256_base64"


class HashTypedDict(TypedDict):
    r"""Provide either `hash` or `plaintext`"""

    value: str
    r"""The hashed and encoded key"""
    variant: Variant
    r"""The algorithm for hashing and encoding, currently only sha256 and base64 are supported"""


class Hash(BaseModel):
    r"""Provide either `hash` or `plaintext`"""

    value: str
    r"""The hashed and encoded key"""

    variant: Variant
    r"""The algorithm for hashing and encoding, currently only sha256 and base64 are supported"""


class V1MigrationsCreateKeysInterval(str, Enum):
    r"""Unkey will automatically refill verifications at the set interval."""

    DAILY = "daily"
    MONTHLY = "monthly"


class V1MigrationsCreateKeysRefillTypedDict(TypedDict):
    r"""Unkey enables you to refill verifications for each key at regular intervals."""

    interval: V1MigrationsCreateKeysInterval
    r"""Unkey will automatically refill verifications at the set interval."""
    amount: int
    r"""The number of verifications to refill for each occurrence is determined individually for each key."""
    refill_day: NotRequired[float]
    r"""The day verifications will refill each month, when interval is set to 'monthly'"""


class V1MigrationsCreateKeysRefill(BaseModel):
    r"""Unkey enables you to refill verifications for each key at regular intervals."""

    interval: V1MigrationsCreateKeysInterval
    r"""Unkey will automatically refill verifications at the set interval."""

    amount: int
    r"""The number of verifications to refill for each occurrence is determined individually for each key."""

    refill_day: Annotated[Optional[float], pydantic.Field(alias="refillDay")] = None
    r"""The day verifications will refill each month, when interval is set to 'monthly'"""


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class V1MigrationsCreateKeysType(str, Enum):
    r"""Fast ratelimiting doesn't add latency, while consistent ratelimiting is more accurate.
    https://unkey.dev/docs/features/ratelimiting - Learn more
    """

    FAST = "fast"
    CONSISTENT = "consistent"


class V1MigrationsCreateKeysRatelimitTypedDict(TypedDict):
    r"""Unkey comes with per-key ratelimiting out of the box."""

    limit: int
    r"""The total amount of burstable requests."""
    refill_rate: int
    r"""How many tokens to refill during each refillInterval."""
    refill_interval: int
    r"""Determines the speed at which tokens are refilled, in milliseconds."""
    async_: NotRequired[bool]
    r"""Async will return a response immediately, lowering latency at the cost of accuracy.
    https://unkey.dev/docs/features/ratelimiting - Learn more
    """
    type: NotRequired[V1MigrationsCreateKeysType]
    r"""Fast ratelimiting doesn't add latency, while consistent ratelimiting is more accurate.
    https://unkey.dev/docs/features/ratelimiting - Learn more
    """


class V1MigrationsCreateKeysRatelimit(BaseModel):
    r"""Unkey comes with per-key ratelimiting out of the box."""

    limit: int
    r"""The total amount of burstable requests."""

    refill_rate: Annotated[
        int,
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible.",
            alias="refillRate",
        ),
    ]
    r"""How many tokens to refill during each refillInterval."""

    refill_interval: Annotated[
        int,
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible.",
            alias="refillInterval",
        ),
    ]
    r"""Determines the speed at which tokens are refilled, in milliseconds."""

    async_: Annotated[Optional[bool], pydantic.Field(alias="async")] = False
    r"""Async will return a response immediately, lowering latency at the cost of accuracy.
    https://unkey.dev/docs/features/ratelimiting - Learn more
    """

    type: Annotated[
        Optional[V1MigrationsCreateKeysType],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = V1MigrationsCreateKeysType.FAST
    r"""Fast ratelimiting doesn't add latency, while consistent ratelimiting is more accurate.
    https://unkey.dev/docs/features/ratelimiting - Learn more
    """


class RequestBodyTypedDict(TypedDict):
    api_id: str
    r"""Choose an `API` where this key should be created."""
    prefix: NotRequired[str]
    r"""To make it easier for your users to understand which product an api key belongs to, you can add prefix them.

    For example Stripe famously prefixes their customer ids with cus_ or their api keys with sk_live_.

    The underscore is automatically added if you are defining a prefix, for example: \"prefix\": \"abc\" will result in a key like abc_xxxxxxxxx

    """
    name: NotRequired[str]
    r"""The name for your Key. This is not customer facing."""
    plaintext: NotRequired[str]
    r"""The raw key in plaintext. If provided, unkey encrypts this value and stores it securely. Provide either `hash` or `plaintext`"""
    hash: NotRequired[HashTypedDict]
    r"""Provide either `hash` or `plaintext`"""
    start: NotRequired[str]
    r"""The first 4 characters of the key. If a prefix is used, it should be the prefix plus 4 characters."""
    owner_id: NotRequired[str]
    r"""Deprecated, use `externalId`"""
    external_id: NotRequired[str]
    r"""Your user’s Id. This will provide a link between Unkey and your customer record.
    When validating a key, we will return this back to you, so you can clearly identify your user from their api key.
    """
    meta: NotRequired[Dict[str, Nullable[Any]]]
    r"""This is a place for dynamic meta data, anything that feels useful for you should go here"""
    roles: NotRequired[List[str]]
    r"""A list of roles that this key should have. If the role does not exist, an error is thrown"""
    permissions: NotRequired[List[str]]
    r"""A list of permissions that this key should have. If the permission does not exist, an error is thrown"""
    expires: NotRequired[int]
    r"""You can auto expire keys by providing a unix timestamp in milliseconds. Once Keys expire they will automatically be disabled and are no longer valid unless you enable them again."""
    remaining: NotRequired[int]
    r"""You can limit the number of requests a key can make. Once a key reaches 0 remaining requests, it will automatically be disabled and is no longer valid unless you update it.
    https://unkey.dev/docs/features/remaining - Learn more
    """
    refill: NotRequired[V1MigrationsCreateKeysRefillTypedDict]
    r"""Unkey enables you to refill verifications for each key at regular intervals."""
    ratelimit: NotRequired[V1MigrationsCreateKeysRatelimitTypedDict]
    r"""Unkey comes with per-key ratelimiting out of the box."""
    enabled: NotRequired[bool]
    r"""Sets if key is enabled or disabled. Disabled keys are not valid."""
    environment: NotRequired[str]
    r"""Environments allow you to divide your keyspace.

    Some applications like Stripe, Clerk, WorkOS and others have a concept of \"live\" and \"test\" keys to
    give the developer a way to develop their own application without the risk of modifying real world
    resources.

    When you set an environment, we will return it back to you when validating the key, so you can
    handle it correctly.

    """


class RequestBody(BaseModel):
    api_id: Annotated[str, pydantic.Field(alias="apiId")]
    r"""Choose an `API` where this key should be created."""

    prefix: Optional[str] = None
    r"""To make it easier for your users to understand which product an api key belongs to, you can add prefix them.

    For example Stripe famously prefixes their customer ids with cus_ or their api keys with sk_live_.

    The underscore is automatically added if you are defining a prefix, for example: \"prefix\": \"abc\" will result in a key like abc_xxxxxxxxx

    """

    name: Optional[str] = None
    r"""The name for your Key. This is not customer facing."""

    plaintext: Optional[str] = None
    r"""The raw key in plaintext. If provided, unkey encrypts this value and stores it securely. Provide either `hash` or `plaintext`"""

    hash: Optional[Hash] = None
    r"""Provide either `hash` or `plaintext`"""

    start: Optional[str] = None
    r"""The first 4 characters of the key. If a prefix is used, it should be the prefix plus 4 characters."""

    owner_id: Annotated[
        Optional[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible.",
            alias="ownerId",
        ),
    ] = None
    r"""Deprecated, use `externalId`"""

    external_id: Annotated[Optional[str], pydantic.Field(alias="externalId")] = None
    r"""Your user’s Id. This will provide a link between Unkey and your customer record.
    When validating a key, we will return this back to you, so you can clearly identify your user from their api key.
    """

    meta: Optional[Dict[str, Nullable[Any]]] = None
    r"""This is a place for dynamic meta data, anything that feels useful for you should go here"""

    roles: Optional[List[str]] = None
    r"""A list of roles that this key should have. If the role does not exist, an error is thrown"""

    permissions: Optional[List[str]] = None
    r"""A list of permissions that this key should have. If the permission does not exist, an error is thrown"""

    expires: Optional[int] = None
    r"""You can auto expire keys by providing a unix timestamp in milliseconds. Once Keys expire they will automatically be disabled and are no longer valid unless you enable them again."""

    remaining: Optional[int] = None
    r"""You can limit the number of requests a key can make. Once a key reaches 0 remaining requests, it will automatically be disabled and is no longer valid unless you update it.
    https://unkey.dev/docs/features/remaining - Learn more
    """

    refill: Optional[V1MigrationsCreateKeysRefill] = None
    r"""Unkey enables you to refill verifications for each key at regular intervals."""

    ratelimit: Optional[V1MigrationsCreateKeysRatelimit] = None
    r"""Unkey comes with per-key ratelimiting out of the box."""

    enabled: Optional[bool] = True
    r"""Sets if key is enabled or disabled. Disabled keys are not valid."""

    environment: Optional[str] = None
    r"""Environments allow you to divide your keyspace.

    Some applications like Stripe, Clerk, WorkOS and others have a concept of \"live\" and \"test\" keys to
    give the developer a way to develop their own application without the risk of modifying real world
    resources.

    When you set an environment, we will return it back to you when validating the key, so you can
    handle it correctly.

    """


class V1MigrationsCreateKeysResponseBodyTypedDict(TypedDict):
    r"""The key ids of all created keys"""

    key_ids: List[str]
    r"""The ids of the keys. This is not a secret and can be stored as a reference if you wish. You need the keyId to update or delete a key later."""


class V1MigrationsCreateKeysResponseBody(BaseModel):
    r"""The key ids of all created keys"""

    key_ids: Annotated[List[str], pydantic.Field(alias="keyIds")]
    r"""The ids of the keys. This is not a secret and can be stored as a reference if you wish. You need the keyId to update or delete a key later."""


class V1MigrationsCreateKeysResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[V1MigrationsCreateKeysResponseBodyTypedDict]
    r"""The key ids of all created keys"""


class V1MigrationsCreateKeysResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[V1MigrationsCreateKeysResponseBody] = None
    r"""The key ids of all created keys"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
from enum import Enum
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict, deprecated
from unkey_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class UpdateKeyType(str, Enum):
    r"""Fast ratelimiting doesn't add latency, while consistent ratelimiting is more accurate.
    Deprecated, use 'async' instead
    https://unkey.dev/docs/features/ratelimiting - Learn more
    """

    FAST = "fast"
    CONSISTENT = "consistent"


class UpdateKeyRatelimitTypedDict(TypedDict):
    r"""Unkey comes with per-key ratelimiting out of the box. Set `null` to disable."""

    limit: int
    r"""The total amount of requests allowed in a single window."""
    type: NotRequired[UpdateKeyType]
    r"""Fast ratelimiting doesn't add latency, while consistent ratelimiting is more accurate.
    Deprecated, use 'async' instead
    https://unkey.dev/docs/features/ratelimiting - Learn more
    """
    async_: NotRequired[bool]
    r"""Asnyc ratelimiting doesn't add latency, while sync ratelimiting is slightly more accurate.
    https://unkey.dev/docs/features/ratelimiting - Learn more
    """
    refill_rate: NotRequired[int]
    r"""How many tokens to refill during each refillInterval.
    Deprecated, use 'limit' instead.
    """
    refill_interval: NotRequired[int]
    r"""Determines the speed at which tokens are refilled, in milliseconds.
    Deprecated, use 'duration'
    """
    duration: NotRequired[int]
    r"""The duration of each ratelimit window, in milliseconds.
    This field will become required in a future version.
    """


class UpdateKeyRatelimit(BaseModel):
    r"""Unkey comes with per-key ratelimiting out of the box. Set `null` to disable."""

    limit: int
    r"""The total amount of requests allowed in a single window."""

    type: Annotated[
        Optional[UpdateKeyType],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = None
    r"""Fast ratelimiting doesn't add latency, while consistent ratelimiting is more accurate.
    Deprecated, use 'async' instead
    https://unkey.dev/docs/features/ratelimiting - Learn more
    """

    async_: Annotated[Optional[bool], pydantic.Field(alias="async")] = False
    r"""Asnyc ratelimiting doesn't add latency, while sync ratelimiting is slightly more accurate.
    https://unkey.dev/docs/features/ratelimiting - Learn more
    """

    refill_rate: Annotated[
        Optional[int],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible.",
            alias="refillRate",
        ),
    ] = None
    r"""How many tokens to refill during each refillInterval.
    Deprecated, use 'limit' instead.
    """

    refill_interval: Annotated[
        Optional[int],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible.",
            alias="refillInterval",
        ),
    ] = None
    r"""Determines the speed at which tokens are refilled, in milliseconds.
    Deprecated, use 'duration'
    """

    duration: Optional[int] = None
    r"""The duration of each ratelimit window, in milliseconds.
    This field will become required in a future version.
    """


class UpdateKeyInterval(str, Enum):
    r"""Unkey will automatically refill verifications at the set interval. If null is used the refill functionality will be removed from the key."""

    DAILY = "daily"
    MONTHLY = "monthly"


class UpdateKeyRefillTypedDict(TypedDict):
    r"""Unkey enables you to refill verifications for each key at regular intervals."""

    interval: UpdateKeyInterval
    r"""Unkey will automatically refill verifications at the set interval. If null is used the refill functionality will be removed from the key."""
    amount: int
    r"""The amount of verifications to refill for each occurrence is determined individually for each key."""
    refill_day: NotRequired[float]
    r"""The day verifications will refill each month, when interval is set to 'monthly'"""


class UpdateKeyRefill(BaseModel):
    r"""Unkey enables you to refill verifications for each key at regular intervals."""

    interval: UpdateKeyInterval
    r"""Unkey will automatically refill verifications at the set interval. If null is used the refill functionality will be removed from the key."""

    amount: int
    r"""The amount of verifications to refill for each occurrence is determined individually for each key."""

    refill_day: Annotated[Optional[float], pydantic.Field(alias="refillDay")] = None
    r"""The day verifications will refill each month, when interval is set to 'monthly'"""


class RolesTypedDict(TypedDict):
    id: NotRequired[str]
    r"""The id of the role. Provide either `id` or `name`. If both are provided `id` is used."""
    name: NotRequired[str]
    r"""Identify the role via its name. Provide either `id` or `name`. If both are provided `id` is used."""
    create: NotRequired[bool]
    r"""Set to true to automatically create the permissions they do not exist yet. Only works when specifying `name`.
    Autocreating roles requires your root key to have the `rbac.*.create_role` permission, otherwise the request will get rejected
    """


class Roles(BaseModel):
    id: Optional[str] = None
    r"""The id of the role. Provide either `id` or `name`. If both are provided `id` is used."""

    name: Optional[str] = None
    r"""Identify the role via its name. Provide either `id` or `name`. If both are provided `id` is used."""

    create: Optional[bool] = None
    r"""Set to true to automatically create the permissions they do not exist yet. Only works when specifying `name`.
    Autocreating roles requires your root key to have the `rbac.*.create_role` permission, otherwise the request will get rejected
    """


class UpdateKeyPermissionsTypedDict(TypedDict):
    id: NotRequired[str]
    r"""The id of the permission. Provide either `id` or `name`. If both are provided `id` is used."""
    name: NotRequired[str]
    r"""Identify the permission via its name. Provide either `id` or `name`. If both are provided `id` is used."""
    create: NotRequired[bool]
    r"""Set to true to automatically create the permissions they do not exist yet. Only works when specifying `name`.
    Autocreating permissions requires your root key to have the `rbac.*.create_permission` permission, otherwise the request will get rejected
    """


class UpdateKeyPermissions(BaseModel):
    id: Optional[str] = None
    r"""The id of the permission. Provide either `id` or `name`. If both are provided `id` is used."""

    name: Optional[str] = None
    r"""Identify the permission via its name. Provide either `id` or `name`. If both are provided `id` is used."""

    create: Optional[bool] = None
    r"""Set to true to automatically create the permissions they do not exist yet. Only works when specifying `name`.
    Autocreating permissions requires your root key to have the `rbac.*.create_permission` permission, otherwise the request will get rejected
    """


class UpdateKeyRequestBodyTypedDict(TypedDict):
    r"""Update a key's configuration.
    The `apis.<API_ID>.update_key` permission is required.
    """

    key_id: str
    r"""The id of the key you want to modify"""
    name: NotRequired[Nullable[str]]
    r"""The name of the key"""
    owner_id: NotRequired[Nullable[str]]
    r"""Deprecated, use `externalId`
    The id of the tenant associated with this key. Use whatever reference you have in your system to identify the tenant. When verifying the key, we will send this field back to you, so you know who is accessing your API.
    """
    external_id: NotRequired[Nullable[str]]
    r"""The id of the tenant associated with this key. Use whatever reference you have in your system to identify the tenant. When verifying the key, we will send this back to you, so you know who is accessing your API.
    Under the hood this upserts and connects an `ìdentity` for you.
    To disconnect the key from an identity, set `externalId: null`.
    """
    meta: NotRequired[Nullable[Dict[str, Nullable[Any]]]]
    r"""Any additional metadata you want to store with the key"""
    expires: NotRequired[Nullable[int]]
    r"""The unix timestamp in milliseconds when the key will expire. If this field is null or undefined, the key is not expiring."""
    ratelimit: NotRequired[Nullable[UpdateKeyRatelimitTypedDict]]
    r"""Unkey comes with per-key ratelimiting out of the box. Set `null` to disable."""
    remaining: NotRequired[Nullable[int]]
    r"""The number of requests that can be made with this key before it becomes invalid. Set `null` to disable."""
    refill: NotRequired[Nullable[UpdateKeyRefillTypedDict]]
    r"""Unkey enables you to refill verifications for each key at regular intervals."""
    enabled: NotRequired[bool]
    r"""Set if key is enabled or disabled. If disabled, the key cannot be used to verify."""
    roles: NotRequired[List[RolesTypedDict]]
    r"""The roles you want to set for this key. This overwrites all existing roles.
    Setting roles requires the `rbac.*.add_role_to_key` permission.
    """
    permissions: NotRequired[List[UpdateKeyPermissionsTypedDict]]
    r"""The permissions you want to set for this key. This overwrites all existing permissions.
    Setting permissions requires the `rbac.*.add_permission_to_key` permission.
    """


class UpdateKeyRequestBody(BaseModel):
    r"""Update a key's configuration.
    The `apis.<API_ID>.update_key` permission is required.
    """

    key_id: Annotated[str, pydantic.Field(alias="keyId")]
    r"""The id of the key you want to modify"""

    name: OptionalNullable[str] = UNSET
    r"""The name of the key"""

    owner_id: Annotated[
        OptionalNullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible.",
            alias="ownerId",
        ),
    ] = UNSET
    r"""Deprecated, use `externalId`
    The id of the tenant associated with this key. Use whatever reference you have in your system to identify the tenant. When verifying the key, we will send this field back to you, so you know who is accessing your API.
    """

    external_id: Annotated[
        OptionalNullable[str], pydantic.Field(alias="externalId")
    ] = UNSET
    r"""The id of the tenant associated with this key. Use whatever reference you have in your system to identify the tenant. When verifying the key, we will send this back to you, so you know who is accessing your API.
    Under the hood this upserts and connects an `ìdentity` for you.
    To disconnect the key from an identity, set `externalId: null`.
    """

    meta: OptionalNullable[Dict[str, Nullable[Any]]] = UNSET
    r"""Any additional metadata you want to store with the key"""

    expires: OptionalNullable[int] = UNSET
    r"""The unix timestamp in milliseconds when the key will expire. If this field is null or undefined, the key is not expiring."""

    ratelimit: OptionalNullable[UpdateKeyRatelimit] = UNSET
    r"""Unkey comes with per-key ratelimiting out of the box. Set `null` to disable."""

    remaining: OptionalNullable[int] = UNSET
    r"""The number of requests that can be made with this key before it becomes invalid. Set `null` to disable."""

    refill: OptionalNullable[UpdateKeyRefill] = UNSET
    r"""Unkey enables you to refill verifications for each key at regular intervals."""

    enabled: Optional[bool] = None
    r"""Set if key is enabled or disabled. If disabled, the key cannot be used to verify."""

    roles: Optional[List[Roles]] = None
    r"""The roles you want to set for this key. This overwrites all existing roles.
    Setting roles requires the `rbac.*.add_role_to_key` permission.
    """

    permissions: Optional[List[UpdateKeyPermissions]] = None
    r"""The permissions you want to set for this key. This overwrites all existing permissions.
    Setting permissions requires the `rbac.*.add_permission_to_key` permission.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "name",
            "ownerId",
            "externalId",
            "meta",
            "expires",
            "ratelimit",
            "remaining",
            "refill",
            "enabled",
            "roles",
            "permissions",
        ]
        nullable_fields = [
            "name",
            "ownerId",
            "externalId",
            "meta",
            "expires",
            "ratelimit",
            "remaining",
            "refill",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class UpdateKeyResponseBodyTypedDict(TypedDict):
    r"""The key was successfully updated, it may take up to 30s for this to take effect in all regions"""


class UpdateKeyResponseBody(BaseModel):
    r"""The key was successfully updated, it may take up to 30s for this to take effect in all regions"""


class UpdateKeyResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[UpdateKeyResponseBodyTypedDict]
    r"""The key was successfully updated, it may take up to 30s for this to take effect in all regions"""


class UpdateKeyResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[UpdateKeyResponseBody] = None
    r"""The key was successfully updated, it may take up to 30s for this to take effect in all regions"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from typing import Callable, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict
from unkey_py.types import BaseModel
from unkey_py.utils import FieldMetadata, QueryParamMetadata


class ListIdentitiesRequestTypedDict(TypedDict):
    environment: NotRequired[str]
    limit: NotRequired[int]
    cursor: NotRequired[str]


class ListIdentitiesRequest(BaseModel):
    environment: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = "default"

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 100

    cursor: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None


class ListIdentitiesRatelimitsTypedDict(TypedDict):
    name: str
    r"""The name of this limit. You will need to use this again when verifying a key."""
    limit: int
    r"""How many requests may pass within a given window before requests are rejected."""
    duration: int
    r"""The duration for each ratelimit window in milliseconds."""


class ListIdentitiesRatelimits(BaseModel):
    name: str
    r"""The name of this limit. You will need to use this again when verifying a key."""

    limit: int
    r"""How many requests may pass within a given window before requests are rejected."""

    duration: int
    r"""The duration for each ratelimit window in milliseconds."""


class ListIdentitiesIdentitiesTypedDict(TypedDict):
    id: str
    r"""The id of this identity. Used to interact with unkey's API"""
    external_id: str
    r"""The id in your system"""
    ratelimits: List[ListIdentitiesRatelimitsTypedDict]
    r"""When verifying keys, you can specify which limits you want to use and all keys attached to this identity, will share the limits."""


class ListIdentitiesIdentities(BaseModel):
    id: str
    r"""The id of this identity. Used to interact with unkey's API"""

    external_id: Annotated[str, pydantic.Field(alias="externalId")]
    r"""The id in your system"""

    ratelimits: List[ListIdentitiesRatelimits]
    r"""When verifying keys, you can specify which limits you want to use and all keys attached to this identity, will share the limits."""


class ListIdentitiesResponseBodyTypedDict(TypedDict):
    r"""A list of identities."""

    identities: List[ListIdentitiesIdentitiesTypedDict]
    r"""A list of identities."""
    total: int
    r"""The total number of identities for this environment"""
    cursor: NotRequired[str]
    r"""The cursor to use for the next page of results, if no cursor is returned, there are no more results"""


class ListIdentitiesResponseBody(BaseModel):
    r"""A list of identities."""

    identities: List[ListIdentitiesIdentities]
    r"""A list of identities."""

    total: int
    r"""The total number of identities for this environment"""

    cursor: Optional[str] = None
    r"""The cursor to use for the next page of results, if no cursor is returned, there are no more results"""


class ListIdentitiesResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[ListIdentitiesResponseBodyTypedDict]
    r"""A list of identities."""


class ListIdentitiesResponse(BaseModel):
    next: Callable[[], Optional[ListIdentitiesResponse]]

    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[ListIdentitiesResponseBody] = None
    r"""A list of identities."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .permissionquery import PermissionQuery, PermissionQueryTypedDict
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict, deprecated
from unkey_py.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL


class AuthorizationTypedDict(TypedDict):
    r"""Perform RBAC checks"""

    permissions: NotRequired[Nullable[PermissionQueryTypedDict]]
    r"""A query for which permissions you require"""


class Authorization(BaseModel):
    r"""Perform RBAC checks"""

    permissions: OptionalNullable[PermissionQuery] = UNSET
    r"""A query for which permissions you require"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["permissions"]
        nullable_fields = ["permissions"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class RemainingTypedDict(TypedDict):
    r"""Customize the behaviour of deducting remaining uses. When some of your endpoints are more expensive than others, you can set a custom `cost` for each."""

    cost: NotRequired[int]
    r"""How many tokens should be deducted from the current `remaining` value. Set it to 0, to make it free."""


class Remaining(BaseModel):
    r"""Customize the behaviour of deducting remaining uses. When some of your endpoints are more expensive than others, you can set a custom `cost` for each."""

    cost: Optional[int] = 1
    r"""How many tokens should be deducted from the current `remaining` value. Set it to 0, to make it free."""


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class V1KeysVerifyKeyRequestRatelimitTypedDict(TypedDict):
    r"""Use 'ratelimits' with `[{ name: \"default\", cost: 2}]`"""

    cost: NotRequired[int]
    r"""Override how many tokens are deducted during the ratelimit operation."""


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class V1KeysVerifyKeyRequestRatelimit(BaseModel):
    r"""Use 'ratelimits' with `[{ name: \"default\", cost: 2}]`"""

    cost: Optional[int] = 1
    r"""Override how many tokens are deducted during the ratelimit operation."""


class RatelimitsModelTypedDict(TypedDict):
    name: str
    r"""The name of the ratelimit."""
    cost: NotRequired[int]
    r"""Optionally override how expensive this operation is and how many tokens are deducted from the current limit."""
    limit: NotRequired[int]
    r"""Optionally override the limit."""
    duration: NotRequired[int]
    r"""Optionally override the ratelimit window duration."""


class RatelimitsModel(BaseModel):
    name: str
    r"""The name of the ratelimit."""

    cost: Optional[int] = 1
    r"""Optionally override how expensive this operation is and how many tokens are deducted from the current limit."""

    limit: Optional[int] = None
    r"""Optionally override the limit."""

    duration: Optional[int] = None
    r"""Optionally override the ratelimit window duration."""


class V1KeysVerifyKeyRequestTypedDict(TypedDict):
    key: str
    r"""The key to verify"""
    api_id: NotRequired[str]
    r"""The id of the api where the key belongs to. This is optional for now but will be required soon.
    The key will be verified against the api's configuration. If the key does not belong to the api, the verification will fail.
    """
    tags: NotRequired[List[str]]
    r"""Tags do not influence the outcome of a verification.
    They can be added to filter or aggregate historical verification data for your analytics needs.
    To unkey, a tag is simply a string, we don't enforce any schema but leave that up to you.
    The only exception is that each tag must be between 1 and 128 characters long.
    A typical setup would be to add key-value pairs of resources or locations, that you need later when querying.

    """
    authorization: NotRequired[AuthorizationTypedDict]
    r"""Perform RBAC checks"""
    remaining: NotRequired[RemainingTypedDict]
    r"""Customize the behaviour of deducting remaining uses. When some of your endpoints are more expensive than others, you can set a custom `cost` for each."""
    ratelimit: NotRequired[V1KeysVerifyKeyRequestRatelimitTypedDict]
    r"""Use 'ratelimits' with `[{ name: \"default\", cost: 2}]`"""
    ratelimits: NotRequired[List[RatelimitsModelTypedDict]]
    r"""You can check against multiple ratelimits when verifying a key. Let's say you are building an app that uses AI under the hood and you want to limit your customers to 500 requests per hour, but also ensure they use up less than 20k tokens per day.

    https://www.unkey.com/docs/concepts/identities/ratelimits
    """


class V1KeysVerifyKeyRequest(BaseModel):
    key: str
    r"""The key to verify"""

    api_id: Annotated[Optional[str], pydantic.Field(alias="apiId")] = None
    r"""The id of the api where the key belongs to. This is optional for now but will be required soon.
    The key will be verified against the api's configuration. If the key does not belong to the api, the verification will fail.
    """

    tags: Optional[List[str]] = None
    r"""Tags do not influence the outcome of a verification.
    They can be added to filter or aggregate historical verification data for your analytics needs.
    To unkey, a tag is simply a string, we don't enforce any schema but leave that up to you.
    The only exception is that each tag must be between 1 and 128 characters long.
    A typical setup would be to add key-value pairs of resources or locations, that you need later when querying.

    """

    authorization: Optional[Authorization] = None
    r"""Perform RBAC checks"""

    remaining: Optional[Remaining] = None
    r"""Customize the behaviour of deducting remaining uses. When some of your endpoints are more expensive than others, you can set a custom `cost` for each."""

    ratelimit: Annotated[
        Optional[V1KeysVerifyKeyRequestRatelimit],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = None
    r"""Use 'ratelimits' with `[{ name: \"default\", cost: 2}]`"""

    ratelimits: Optional[List[RatelimitsModel]] = None
    r"""You can check against multiple ratelimits when verifying a key. Let's say you are building an app that uses AI under the hood and you want to limit your customers to 500 requests per hour, but also ensure they use up less than 20k tokens per day.

    https://www.unkey.com/docs/concepts/identities/ratelimits
    """



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict
from unkey_py.types import BaseModel


class CreateAPIRequestBodyTypedDict(TypedDict):
    name: str
    r"""The name for your API. This is not customer facing."""


class CreateAPIRequestBody(BaseModel):
    name: str
    r"""The name for your API. This is not customer facing."""


class CreateAPIResponseBodyTypedDict(TypedDict):
    r"""The configuration for an api"""

    api_id: str
    r"""The id of the api"""


class CreateAPIResponseBody(BaseModel):
    r"""The configuration for an api"""

    api_id: Annotated[str, pydantic.Field(alias="apiId")]
    r"""The id of the api"""


class CreateAPIResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[CreateAPIResponseBodyTypedDict]
    r"""The configuration for an api"""


class CreateAPIResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[CreateAPIResponseBody] = None
    r"""The configuration for an api"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict
from unkey_py.types import BaseModel


class DeletePermissionRequestBodyTypedDict(TypedDict):
    permission_id: str
    r"""The id of the permission you want to delete."""


class DeletePermissionRequestBody(BaseModel):
    permission_id: Annotated[str, pydantic.Field(alias="permissionId")]
    r"""The id of the permission you want to delete."""


class DeletePermissionResponseBodyTypedDict(TypedDict):
    r"""Sucessfully deleted a permission"""


class DeletePermissionResponseBody(BaseModel):
    r"""Sucessfully deleted a permission"""


class DeletePermissionResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[DeletePermissionResponseBodyTypedDict]
    r"""Sucessfully deleted a permission"""


class DeletePermissionResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[DeletePermissionResponseBody] = None
    r"""Sucessfully deleted a permission"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from typing import Any, Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict
from unkey_py.types import BaseModel, Nullable
from unkey_py.utils import FieldMetadata, QueryParamMetadata


class GetIdentityRequestTypedDict(TypedDict):
    identity_id: NotRequired[str]
    external_id: NotRequired[str]


class GetIdentityRequest(BaseModel):
    identity_id: Annotated[
        Optional[str],
        pydantic.Field(alias="identityId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    external_id: Annotated[
        Optional[str],
        pydantic.Field(alias="externalId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None


class GetIdentityRatelimitsTypedDict(TypedDict):
    name: str
    r"""The name of this limit. You will need to use this again when verifying a key."""
    limit: int
    r"""How many requests may pass within a given window before requests are rejected."""
    duration: int
    r"""The duration for each ratelimit window in milliseconds."""


class GetIdentityRatelimits(BaseModel):
    name: str
    r"""The name of this limit. You will need to use this again when verifying a key."""

    limit: int
    r"""How many requests may pass within a given window before requests are rejected."""

    duration: int
    r"""The duration for each ratelimit window in milliseconds."""


class GetIdentityResponseBodyTypedDict(TypedDict):
    r"""The configuration for an api"""

    id: str
    r"""The id of this identity. Used to interact with unkey's API"""
    external_id: str
    r"""The id in your system"""
    meta: Dict[str, Nullable[Any]]
    r"""The meta object defined for this identity."""
    ratelimits: List[GetIdentityRatelimitsTypedDict]
    r"""When verifying keys, you can specify which limits you want to use and all keys attached to this identity, will share the limits."""


class GetIdentityResponseBody(BaseModel):
    r"""The configuration for an api"""

    id: str
    r"""The id of this identity. Used to interact with unkey's API"""

    external_id: Annotated[str, pydantic.Field(alias="externalId")]
    r"""The id in your system"""

    meta: Dict[str, Nullable[Any]]
    r"""The meta object defined for this identity."""

    ratelimits: List[GetIdentityRatelimits]
    r"""When verifying keys, you can specify which limits you want to use and all keys attached to this identity, will share the limits."""


class GetIdentityResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[GetIdentityResponseBodyTypedDict]
    r"""The configuration for an api"""


class GetIdentityResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[GetIdentityResponseBody] = None
    r"""The configuration for an api"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict
from unkey_py.types import BaseModel, Nullable, OptionalNullable, UNSET_SENTINEL


class Interval(str, Enum):
    r"""Determines the rate at which verifications will be refilled. When 'daily' is set for 'interval' 'refillDay' will be set to null."""

    DAILY = "daily"
    MONTHLY = "monthly"


class RefillTypedDict(TypedDict):
    r"""Unkey allows you to refill remaining verifications on a key on a regular interval."""

    interval: Interval
    r"""Determines the rate at which verifications will be refilled. When 'daily' is set for 'interval' 'refillDay' will be set to null."""
    amount: int
    r"""Resets `remaining` to this value every interval."""
    refill_day: NotRequired[Nullable[float]]
    r"""The day verifications will refill each month, when interval is set to 'monthly'. Value is not zero-indexed making 1 the first day of the month. If left blank it will default to the first day of the month. When 'daily' is set for 'interval' 'refillDay' will be set to null."""
    last_refill_at: NotRequired[int]
    r"""The unix timestamp in miliseconds when the key was last refilled."""


class Refill(BaseModel):
    r"""Unkey allows you to refill remaining verifications on a key on a regular interval."""

    interval: Interval
    r"""Determines the rate at which verifications will be refilled. When 'daily' is set for 'interval' 'refillDay' will be set to null."""

    amount: int
    r"""Resets `remaining` to this value every interval."""

    refill_day: Annotated[
        OptionalNullable[float], pydantic.Field(alias="refillDay")
    ] = 1
    r"""The day verifications will refill each month, when interval is set to 'monthly'. Value is not zero-indexed making 1 the first day of the month. If left blank it will default to the first day of the month. When 'daily' is set for 'interval' 'refillDay' will be set to null."""

    last_refill_at: Annotated[Optional[int], pydantic.Field(alias="lastRefillAt")] = (
        None
    )
    r"""The unix timestamp in miliseconds when the key was last refilled."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["refillDay", "lastRefillAt"]
        nullable_fields = ["refillDay"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class Type(str, Enum):
    r"""Fast ratelimiting doesn't add latency, while consistent ratelimiting is more accurate.
    https://unkey.dev/docs/features/ratelimiting - Learn more
    """

    FAST = "fast"
    CONSISTENT = "consistent"


class RatelimitModelTypedDict(TypedDict):
    r"""Unkey comes with per-key ratelimiting out of the box."""

    async_: bool
    limit: int
    r"""The total amount of burstable requests."""
    duration: int
    r"""The duration of the ratelimit window, in milliseconds."""
    type: NotRequired[Type]
    r"""Fast ratelimiting doesn't add latency, while consistent ratelimiting is more accurate.
    https://unkey.dev/docs/features/ratelimiting - Learn more
    """
    refill_rate: NotRequired[int]
    r"""How many tokens to refill during each refillInterval."""
    refill_interval: NotRequired[int]
    r"""Determines the speed at which tokens are refilled, in milliseconds."""


class RatelimitModel(BaseModel):
    r"""Unkey comes with per-key ratelimiting out of the box."""

    async_: Annotated[bool, pydantic.Field(alias="async")]

    limit: int
    r"""The total amount of burstable requests."""

    duration: int
    r"""The duration of the ratelimit window, in milliseconds."""

    type: Optional[Type] = None
    r"""Fast ratelimiting doesn't add latency, while consistent ratelimiting is more accurate.
    https://unkey.dev/docs/features/ratelimiting - Learn more
    """

    refill_rate: Annotated[Optional[int], pydantic.Field(alias="refillRate")] = None
    r"""How many tokens to refill during each refillInterval."""

    refill_interval: Annotated[
        Optional[int], pydantic.Field(alias="refillInterval")
    ] = None
    r"""Determines the speed at which tokens are refilled, in milliseconds."""


class IdentityTypedDict(TypedDict):
    r"""The identity of the key"""

    id: str
    r"""The id of the identity"""
    external_id: str
    r"""The external id of the identity"""
    meta: NotRequired[Dict[str, Nullable[Any]]]
    r"""Any additional metadata attached to the identity"""


class Identity(BaseModel):
    r"""The identity of the key"""

    id: str
    r"""The id of the identity"""

    external_id: Annotated[str, pydantic.Field(alias="externalId")]
    r"""The external id of the identity"""

    meta: Optional[Dict[str, Nullable[Any]]] = None
    r"""Any additional metadata attached to the identity"""


class KeyTypedDict(TypedDict):
    id: str
    r"""The id of the key"""
    start: str
    r"""The first few characters of the key to visually identify it"""
    workspace_id: str
    r"""The id of the workspace that owns the key"""
    created_at: int
    r"""The unix timestamp in milliseconds when the key was created"""
    api_id: NotRequired[str]
    r"""The id of the api that this key is for"""
    name: NotRequired[str]
    r"""The name of the key, give keys a name to easily identify their purpose"""
    owner_id: NotRequired[str]
    r"""The id of the tenant associated with this key. Use whatever reference you have in your system to identify the tenant. When verifying the key, we will send this field back to you, so you know who is accessing your API."""
    meta: NotRequired[Dict[str, Nullable[Any]]]
    r"""Any additional metadata you want to store with the key"""
    updated_at: NotRequired[int]
    r"""The unix timestamp in milliseconds when the key was last updated"""
    expires: NotRequired[int]
    r"""The unix timestamp in milliseconds when the key will expire. If this field is null or undefined, the key is not expiring."""
    remaining: NotRequired[int]
    r"""The number of requests that can be made with this key before it becomes invalid. If this field is null or undefined, the key has no request limit."""
    refill: NotRequired[RefillTypedDict]
    r"""Unkey allows you to refill remaining verifications on a key on a regular interval."""
    ratelimit: NotRequired[RatelimitModelTypedDict]
    r"""Unkey comes with per-key ratelimiting out of the box."""
    roles: NotRequired[List[str]]
    r"""All roles this key belongs to"""
    permissions: NotRequired[List[str]]
    r"""All permissions this key has"""
    enabled: NotRequired[bool]
    r"""Sets if key is enabled or disabled. Disabled keys are not valid."""
    plaintext: NotRequired[str]
    r"""The key in plaintext"""
    identity: NotRequired[IdentityTypedDict]
    r"""The identity of the key"""


class Key(BaseModel):
    id: str
    r"""The id of the key"""

    start: str
    r"""The first few characters of the key to visually identify it"""

    workspace_id: Annotated[str, pydantic.Field(alias="workspaceId")]
    r"""The id of the workspace that owns the key"""

    created_at: Annotated[int, pydantic.Field(alias="createdAt")]
    r"""The unix timestamp in milliseconds when the key was created"""

    api_id: Annotated[Optional[str], pydantic.Field(alias="apiId")] = None
    r"""The id of the api that this key is for"""

    name: Optional[str] = None
    r"""The name of the key, give keys a name to easily identify their purpose"""

    owner_id: Annotated[Optional[str], pydantic.Field(alias="ownerId")] = None
    r"""The id of the tenant associated with this key. Use whatever reference you have in your system to identify the tenant. When verifying the key, we will send this field back to you, so you know who is accessing your API."""

    meta: Optional[Dict[str, Nullable[Any]]] = None
    r"""Any additional metadata you want to store with the key"""

    updated_at: Annotated[Optional[int], pydantic.Field(alias="updatedAt")] = None
    r"""The unix timestamp in milliseconds when the key was last updated"""

    expires: Optional[int] = None
    r"""The unix timestamp in milliseconds when the key will expire. If this field is null or undefined, the key is not expiring."""

    remaining: Optional[int] = None
    r"""The number of requests that can be made with this key before it becomes invalid. If this field is null or undefined, the key has no request limit."""

    refill: Optional[Refill] = None
    r"""Unkey allows you to refill remaining verifications on a key on a regular interval."""

    ratelimit: Optional[RatelimitModel] = None
    r"""Unkey comes with per-key ratelimiting out of the box."""

    roles: Optional[List[str]] = None
    r"""All roles this key belongs to"""

    permissions: Optional[List[str]] = None
    r"""All permissions this key has"""

    enabled: Optional[bool] = None
    r"""Sets if key is enabled or disabled. Disabled keys are not valid."""

    plaintext: Optional[str] = None
    r"""The key in plaintext"""

    identity: Optional[Identity] = None
    r"""The identity of the key"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
import pydantic
from typing import Any, Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict
from unkey_py.types import BaseModel, Nullable


class CreateIdentityRatelimitsTypedDict(TypedDict):
    name: str
    r"""The name of this limit. You will need to use this again when verifying a key."""
    limit: int
    r"""How many requests may pass within a given window before requests are rejected."""
    duration: int
    r"""The duration for each ratelimit window in milliseconds."""


class CreateIdentityRatelimits(BaseModel):
    name: str
    r"""The name of this limit. You will need to use this again when verifying a key."""

    limit: int
    r"""How many requests may pass within a given window before requests are rejected."""

    duration: int
    r"""The duration for each ratelimit window in milliseconds."""


class CreateIdentityRequestBodyTypedDict(TypedDict):
    external_id: str
    r"""The id of this identity in your system.

    This usually comes from your authentication provider and could be a userId, organisationId or even an email.
    It does not matter what you use, as long as it uniquely identifies something in your application.

    `externalId`s are unique across your workspace and therefore a `CONFLICT` error is returned when you try to create duplicates.

    """
    meta: NotRequired[Dict[str, Nullable[Any]]]
    r"""Attach metadata to this identity that you need to have access to when verifying a key.

    This will be returned as part of the `verifyKey` response.

    """
    ratelimits: NotRequired[List[CreateIdentityRatelimitsTypedDict]]
    r"""Attach ratelimits to this identity.

    When verifying keys, you can specify which limits you want to use and all keys attached to this identity, will share the limits.
    """


class CreateIdentityRequestBody(BaseModel):
    external_id: Annotated[str, pydantic.Field(alias="externalId")]
    r"""The id of this identity in your system.

    This usually comes from your authentication provider and could be a userId, organisationId or even an email.
    It does not matter what you use, as long as it uniquely identifies something in your application.

    `externalId`s are unique across your workspace and therefore a `CONFLICT` error is returned when you try to create duplicates.

    """

    meta: Optional[Dict[str, Nullable[Any]]] = None
    r"""Attach metadata to this identity that you need to have access to when verifying a key.

    This will be returned as part of the `verifyKey` response.

    """

    ratelimits: Optional[List[CreateIdentityRatelimits]] = None
    r"""Attach ratelimits to this identity.

    When verifying keys, you can specify which limits you want to use and all keys attached to this identity, will share the limits.
    """


class CreateIdentityResponseBodyTypedDict(TypedDict):
    r"""The configuration for an api"""

    identity_id: str
    r"""The id of the identity. Used internally, you do not need to store this."""


class CreateIdentityResponseBody(BaseModel):
    r"""The configuration for an api"""

    identity_id: Annotated[str, pydantic.Field(alias="identityId")]
    r"""The id of the identity. Used internally, you do not need to store this."""


class CreateIdentityResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    object: NotRequired[CreateIdentityResponseBodyTypedDict]
    r"""The configuration for an api"""


class CreateIdentityResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    object: Optional[CreateIdentityResponseBody] = None
    r"""The configuration for an api"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
from .v1keysverifykeyresponse import (
    V1KeysVerifyKeyResponse,
    V1KeysVerifyKeyResponseTypedDict,
)
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict
from unkey_py.types import BaseModel


class VerifyKeyResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    v1_keys_verify_key_response: NotRequired[V1KeysVerifyKeyResponseTypedDict]
    r"""The verification result"""


class VerifyKeyResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    v1_keys_verify_key_response: Optional[V1KeysVerifyKeyResponse] = None
    r"""The verification result"""
